library(unmarked)

analysis = function(dataset, nSeason, n_vis) {

### Prepare data for 'unmarked', multi-season colext model 
### (MacKenzie et al. 2003)

# Matrix for detection/non-detection data
yMat <- dataset[,1:(nSeason*n_vis)]

# Matrix for seasonal site covariate SeasonBA
SeasonBAMat <- dataset[,((nSeason*n_vis)+2):((nSeason*n_vis)+nSeason+1)]

# Matrix for seasonal site covariate Season
SeasonMat <- dataset[,((nSeason*n_vis)+nSeason+2):((nSeason*n_vis)+
2*nSeason+1)]

# Matrix for seasonal site covariate SeasonSiteTreat
SSTMat <- dataset[,((nSeason*n_vis)+2*nSeason+2):((nSeason*n_vis)+
3*nSeason+1)]

# Combine all the seasonal site covariates into a dataframe
Seasonal <- data.frame(
	SeasonBA = matrix(t(SeasonBAMat), ncol=1),
	Season = matrix(t(SeasonMat), ncol=1),
SeasonSiteTreat = matrix(t(SSTMat), ncol=1))
 

# prepare the unmarked multi-year colext dataframe
oUMF <- unmarkedMultFrame(
	y = yMat,
	siteCovs = dataset[,"SiteTreat", drop=F],
	yearlySiteCovs = Seasonal,
  	numPrimary=nSeason)

### Run models (the order for parameterizing colext models is: 1st season
### occupancy(psi), colonization(gamma), extinction(epsilon), detection(p))

# Models for testing treatment effect on extinction and colonization without # accounting for background differences between Control and Impact sites
Null.fit.1 <- colext(psi1formula = ~1, gammaformula = ~Season, 
epslonformula = ~Season, pformula = ~1, oUMF)
Alt.fit.1 <- colext(~1, ~Season + SeasonSiteTreat, 
~Season + SeasonSiteTreat, ~1, oUMF)


# Models for testing treatment effect on extinction and colonization that
# account for background differences between Control and Impact sites
Null.fit.2 <- colext(~1, ~SiteTreat + Season, 
~SiteTreat + Season, ~1, oUMF)
Alt.fit.2 <- colext(~1, ~SiteTreat + Season + SeasonSiteTreat,
~Sitetreat + Season + SeasonSiteTreat, ~1, oUMF)

# Models for testing treatment effect on extinction and colonization using 
# a conventional BACI interaction term Before-After x Control-Impact
Null.fit.3 <- colext(~1, ~SiteTreat + SeasonBA, 
~SiteTreat + SeasonBA, ~1, oUMF)
Alt.fit.3 <- colext(~1, ~SiteTreat  SeasonBA,
~Sitetreat  SeasonBA, ~1, oUMF)

# Models for testing treatment effects on colonization, extinction, and 
# detection; ### IMPORTANT ###: variable SeasonSiteTreat for testing
# treatment effects on detection has n = nSeason valid columns (i.e., it is 
# unaffected by transitions between seasons). This matrix is designed similar 
# to SeasonSiteTreat used for modeling colonization and extinction, but with
# an extra CONTROL column (as the first column), thus eliminating the need
# for the Null attribute (last column)
Null.fit.4 <- colext(~1, ~SiteTreat + Season, 
~SiteTreat + Season, ~1, oUMF)
Alt.fit.4 <- colext(~1, ~SiteTreat + Season + SeasonSiteTreat,
~Sitetreat + Season + SeasonSiteTreat, ~SeasonSiteTreat, oUMF)

# Combine the model results to a list
list(Null.fit.1 = Null.fit.1, Alt.fit.1 = Alt.fit.1,
      Null.fit.2 = Null.fit.2, Alt.fit.2 = Alt.fit.2,
Null.fit.3 = Null.fit.3, Alt.fit.3 = Alt.fit.3,
Null.fit.4 = Null.fit.4, Alt.fit.4 = Alt.fit.4)

}